# -*- coding: utf-8 -*-

import os
import datetime
import time
import logging
import logging.handlers
import json
import random

import click
from kombu import Connection
from dropshipping.utils.utils import is_asin_valid
from em_tasks.spapi import Spapi
from em_tasks.store.store_util import StoreUtil
from em_tasks.store.models import *
from em_tasks.spree.product_util import ProductUtil

from em_celery import logger, get_offer_service, init_db
from em_celery.tasks.spapi_update_catalog_items_task import spapi_update_catalog_items



@click.command('Send spapi update item offers task to worker.')
@click.option('-s', '--store_code', type=str, required=True, help='Store code.')
@click.option('-b', '--broker_url', type=str, required=True, help='Celery worker broker URL.')
@click.option('-t', '--ttl', type=int, default=36, help='Offer alive hours, default is 36.')
@click.option('-f', '--force', is_flag=True, help='Force to update offers.')
@click.option('-q', '--qps', type=float, help='Quantity per second (QPS) to send task.')
def send_emp_offers_update_task(store_code, broker_url, qps, ttl=36, force=False):
  log_dir = os.path.join(os.path.expanduser('~'), '.em_celery', 'logs')
  if not os.path.isdir(log_dir):
    os.makedirs(log_dir)
  log_path = os.path.join(log_dir, 'amz_offers_update_task_sender.log')
  level = logging.INFO
  max_bytes = 20 * 1024 ** 2
  fh = logging.handlers.RotatingFileHandler(
    log_path, maxBytes=max_bytes, backupCount=5)
  fh.setLevel(level)
  formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
  fh.setFormatter(formatter)
  logger.addHandler(fh)

  init_db()

  data_source = None
  store = StoreUtil.get_store_by_code(store_code)
  platform = store.platform
  if platform == 'Shopify':
    data_source = StoreUtil
  elif platform == 'Spree':
    credential = json.loads(store.api_credential)
    data_source = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))
  else:
    logger.error('[UnsupportedPlatform] Only "Shopify" and "Spree" are supported, %s given', platform)
    return

  if not data_source:
    return

  condition = 'new'
  sender = EmpCatalogUpdateTaskSender(data_source, broker_url, qps, condition, ttl, force)
  sender.run()



class EmpCatalogUpdateTaskSender():
  def __init__(self, data_source, broker_url, qps, condition, ttl, force):
    self.data_source = data_source
    self.offer_service = get_offer_service()
    self.broker_url = broker_url
    self.marketplace = "us"
    self.qps = qps
    self.condition = condition
    self.ttl = ttl
    self.force = force
    self.connection = Connection(broker_url)
    self.queue = 'SpapiCatalogItemsUpdate_US'
    self.last_send_time = None
    self.last_send_time = None
    self.connection = Connection(self.broker_url)
    self.search_opts = {'_source': ['asin', 'time']}

  def get_gerp_asins(self):
    batch_size = 500
    offset = 0
    while True:
        products = (
            GerpSellingProduct.select()
            .limit(batch_size)
            .offset(offset)
        )

        count = 0
        for product in products:
            count += 1
            yield product.asin
        if count < batch_size:
            break

        offset = offset + batch_size

  def run(self):
    asins_buf = []
    batch_size = 500
    # for sp in self.data_source.get_emp_products():
    # asin = sp.get('source_product_id', None) or sp.get('asin', None)
    for asin in self.get_gerp_asins():
      if not is_asin_valid(asin):
        continue

      asins_buf.append(asin)
      if len(asins_buf) < batch_size:
        continue

      self.process_products(asins_buf)
      asins_buf = []

    if asins_buf:
      self.process_products(asins_buf)
      asins_buf = []

  def process_products(self, asins):
    chunks = [asins[x:x + 20] for x in range(0, len(asins), 20)]
    for chunk in chunks:
      spapi_update_catalog_items.apply_async(
          args=(self.marketplace, chunk), queue=self.queue, connection=self.connection
      )
      logger.debug(
        'Added spapi_update_catalog_items(%s, %s)',
        self.marketplace, chunk
      )





if __name__ == "__main__":
  send_emp_offers_update_task()
