# -*- coding: utf-8 -*-

import json
import datetime

import click

from em_tasks.tools.spapi_products_collector import SpapiProductsCollector
from em_tasks.tools.spree.amz_products_filter import AmzProductsFilter
from em_tasks.store.models import *

from em_celery import logger, init_db, get_config, get_spapi, get_product_service


@click.command('Collect Amazon products by Spapi.')
@click.option('-s', '--store_code', required=True, type=str)
@click.option('-m', '--marketplace', required=True, type=str, default='us', help='Amazon marketplace.')
@click.option('-e', '--export', is_flag=True, help='Whether to export products to other countries.')
@click.option('-o', '--output_path', required=True, type=str, help='Output file path')
@click.argument('keywords_path')
def collect_products(keywords_path, store_code, output_path, marketplace='us', export=False):
  init_db()

  output_path = os.path.abspath(os.path.expanduser(output_path))
  output_dir = os.path.dirname(output_path)
  if not os.path.isdir(output_dir):
    os.makedirs(output_dir)

  keywords_path = os.path.abspath(os.path.expanduser(keywords_path))
  if not os.path.isfile(keywords_path):
    logger.error('[KeywordsFileNotFound] %s', keywords_path)
    return

  product_service = get_product_service()
  spapi = get_spapi()

  filter_conds = {'export': export}
  products_filter = AmzProductsFilter(store_code, marketplace, filter_conds)

  keywords = []
  with open(keywords_path, encoding='utf-8', errors='ignore') as fh:
    for line in fh:
      s = line.strip()
      if not s:
        continue

      keywords.append(s)

  output_fh = open(output_path, 'w+', encoding='utf-8', errors='ignore')
  indice_name = 'amz_products_api_{}'.format(marketplace)
  batch_size = 50
  filtered_pids = {}
  filtered_products = {}
  products_collector = SpapiProductsCollector(keywords, spapi, marketplace)
  try:
    for items in products_collector.run():
      result = products_filter.filter_products(items)
      for pid, r in result.items():
        if not pid.startswith('B0'):
          continue

        if not r['passed']:
          continue

        if pid in filtered_pids:
          continue

        filtered_pids[pid] = None
        filtered_products[pid] = items[pid]

        if len(filtered_products) < batch_size:
          continue

        # Save for fetch offer
        pids = list(filtered_products.keys())
        output_fh.write("\n".join(pids))
        output_fh.write("\n")

        # Save to product service
        cur_time = datetime.datetime.strftime(datetime.datetime.utcnow(), '%Y-%m-%dT%H:%M:%S')
        for _, product_info in filtered_products.items():
          product_info['_id'] = product_info['asin']
          product_info['time'] = cur_time

        try:
          product_service.save_products(indice_name, list(filtered_products.values()))
        except Exception as e:
          logger.warning('[ProductSaveToServiceError] %s', filtered_products)
          logger.exception(e)

        filtered_products.clear()

    if filtered_products:
      # Save for fetch offer
      pids = list(filtered_products.keys())
      output_fh.write("\n".join(pids))
      output_fh.write("\n")

      # Save to product service
      cur_time = datetime.datetime.strftime(datetime.datetime.utcnow(), '%Y-%m-%dT%H:%M:%S')
      for _, product_info in filtered_products.items():
        product_info['_id'] = product_info['asin']
        product_info['time'] = cur_time

      try:
        product_service.save_products(indice_name, list(filtered_products.values()))
      except Exception as e:
        logger.warning('[ProductSaveToServiceError] %s', filtered_products)
        logger.exception(e)

      filtered_products.clear()
  finally:
    output_fh.close()


if __name__ == "__main__":
  collect_products()
